// vendor/stackframe/stackframe.ts
function _isNumber(n) {
  return !isNaN(parseFloat(n)) && isFinite(n);
}
function _capitalize(str) {
  return str.charAt(0).toUpperCase() + str.substring(1);
}
function _getter(p) {
  return function() {
    return this[p];
  };
}
function StackFrame(obj) {
  if (!obj)
    return;
  for (var i = 0;i < props.length; i++) {
    if (obj[props[i]] !== undefined) {
      this["set" + _capitalize(props[i])](obj[props[i]]);
    }
  }
}
var booleanProps = ["isConstructor", "isEval", "isNative", "isToplevel"];
var numericProps = ["columnNumber", "lineNumber"];
var stringProps = ["fileName", "functionName", "source"];
var arrayProps = ["args"];
var objectProps = ["evalOrigin"];
var props = booleanProps.concat(numericProps, stringProps, arrayProps, objectProps);
StackFrame.prototype = {
  getArgs: function() {
    return this.args;
  },
  setArgs: function(v) {
    if (Object.prototype.toString.call(v) !== "[object Array]") {
      throw new TypeError("Args must be an Array");
    }
    this.args = v;
  },
  getEvalOrigin: function() {
    return this.evalOrigin;
  },
  setEvalOrigin: function(v) {
    if (v instanceof StackFrame) {
      this.evalOrigin = v;
    } else if (v instanceof Object) {
      this.evalOrigin = new StackFrame(v);
    } else {
      throw new TypeError("Eval Origin must be an Object or StackFrame");
    }
  },
  toString: function() {
    var fileName = this.getFileName() || "";
    var lineNumber = this.getLineNumber() || "";
    var columnNumber = this.getColumnNumber() || "";
    var functionName = this.getFunctionName() || "";
    if (this.getIsEval()) {
      if (fileName) {
        return "[eval] (" + fileName + ":" + lineNumber + ":" + columnNumber + ")";
      }
      return "[eval]:" + lineNumber + ":" + columnNumber;
    }
    if (functionName) {
      return functionName + " (" + fileName + ":" + lineNumber + ":" + columnNumber + ")";
    }
    return fileName + ":" + lineNumber + ":" + columnNumber;
  }
};
StackFrame.fromString = function StackFrame$$fromString(str) {
  var argsStartIndex = str.indexOf("(");
  var argsEndIndex = str.lastIndexOf(")");
  var functionName = str.substring(0, argsStartIndex);
  var args = str.substring(argsStartIndex + 1, argsEndIndex).split(",");
  var locationString = str.substring(argsEndIndex + 1);
  if (locationString.indexOf("@") === 0) {
    var parts = /@(.+?)(?::(\d+))?(?::(\d+))?$/.exec(locationString, "");
    var fileName = parts[1];
    var lineNumber = parts[2];
    var columnNumber = parts[3];
  }
  return new StackFrame({
    functionName,
    args: args || undefined,
    fileName,
    lineNumber: lineNumber || undefined,
    columnNumber: columnNumber || undefined
  });
};
for (i = 0;i < booleanProps.length; i++) {
  StackFrame.prototype["get" + _capitalize(booleanProps[i])] = _getter(booleanProps[i]);
  StackFrame.prototype["set" + _capitalize(booleanProps[i])] = function(p) {
    return function(v) {
      this[p] = Boolean(v);
    };
  }(booleanProps[i]);
}
var i;
for (j = 0;j < numericProps.length; j++) {
  StackFrame.prototype["get" + _capitalize(numericProps[j])] = _getter(numericProps[j]);
  StackFrame.prototype["set" + _capitalize(numericProps[j])] = function(p) {
    return function(v) {
      if (!_isNumber(v)) {
        throw new TypeError(p + " must be a Number");
      }
      this[p] = Number(v);
    };
  }(numericProps[j]);
}
var j;
for (k = 0;k < stringProps.length; k++) {
  StackFrame.prototype["get" + _capitalize(stringProps[k])] = _getter(stringProps[k]);
  StackFrame.prototype["set" + _capitalize(stringProps[k])] = function(p) {
    return function(v) {
      this[p] = String(v);
    };
  }(stringProps[k]);
}
var k;
var stackframe_default = StackFrame;

// vendor/stackframe/error-stack-parser.ts
function ErrorStackParser() {
  var CHROME_IE_STACK_REGEXP = /^\s*at .*(\S+:\d+|\(native\))/m;
  var SAFARI_NATIVE_CODE_REGEXP = /^(eval@)?(\[native code])?$/;
  return {
    parse: function ErrorStackParser$$parse(error) {
      if (error.stack && error.stack.match(CHROME_IE_STACK_REGEXP)) {
        return this.parseV8OrIE(error);
      } else if (error.stack) {
        return this.parseFFOrSafari(error);
      } else {
        throw new Error("Cannot parse given Error object");
      }
    },
    extractLocation: function ErrorStackParser$$extractLocation(urlLike) {
      if (urlLike.indexOf(":") === -1) {
        return [urlLike];
      }
      var regExp = /(.+?)(?::(\d+))?(?::(\d+))?$/;
      var parts = regExp.exec(urlLike.replace(/[()]/g, ""));
      return [parts[1], parts[2] || undefined, parts[3] || undefined];
    },
    parseV8OrIE: function ErrorStackParser$$parseV8OrIE(error) {
      var filtered = error.stack.split("\n").filter(function(line) {
        return !!line.match(CHROME_IE_STACK_REGEXP);
      }, this);
      return filtered.map(function(line) {
        if (line.indexOf("(eval ") > -1) {
          line = line.replace(/eval code/g, "eval").replace(/(\(eval at [^()]*)|(,.*$)/g, "");
        }
        var sanitizedLine = line.replace(/^\s+/, "").replace(/\(eval code/g, "(").replace(/^.*?\s+/, "");
        var location2 = sanitizedLine.match(/ (\(.+\)$)/);
        sanitizedLine = location2 ? sanitizedLine.replace(location2[0], "") : sanitizedLine;
        var locationParts = this.extractLocation(location2 ? location2[1] : sanitizedLine);
        var functionName = location2 && sanitizedLine || undefined;
        var fileName = ["eval", "<anonymous>"].indexOf(locationParts[0]) > -1 ? undefined : locationParts[0];
        return new stackframe_default({
          functionName,
          fileName,
          lineNumber: locationParts[1],
          columnNumber: locationParts[2],
          source: line
        });
      }, this);
    },
    parseFFOrSafari: function ErrorStackParser$$parseFFOrSafari(error) {
      var filtered = error.stack.split("\n").filter(function(line) {
        return !line.match(SAFARI_NATIVE_CODE_REGEXP);
      }, this);
      return filtered.map(function(line) {
        if (line.indexOf(" > eval") > -1) {
          line = line.replace(/ line (\d+)(?: > eval line \d+)* > eval:\d+:\d+/g, ":$1");
        }
        if (line.indexOf("@") === -1 && line.indexOf(":") === -1) {
          return new stackframe_default({
            functionName: line
          });
        } else {
          var functionNameRegex = /((.*".+"[^@]*)?[^@]*)(?:@)/;
          var matches = line.match(functionNameRegex);
          var functionName = matches && matches[1] ? matches[1] : undefined;
          var locationParts = this.extractLocation(line.replace(functionNameRegex, ""));
          return new stackframe_default({
            functionName,
            fileName: locationParts[0],
            lineNumber: locationParts[1],
            columnNumber: locationParts[2],
            source: line
          });
        }
      }, this);
    }
  };
}
var errorStackParser = new ErrorStackParser;
var error_stack_parser_default = errorStackParser;

// environments.ts
var IN_BROWSER = true;
var IN_BROWSER_MAIN_THREAD = IN_BROWSER && typeof window === "object" && typeof document === "object" && typeof document.createElement === "function" && typeof sessionStorage === "object" && typeof importScripts !== "function";
var IN_BROWSER_WEB_WORKER = IN_BROWSER && typeof importScripts === "function" && typeof self === "object";
var IN_SAFARI = typeof navigator === "object" && typeof navigator.userAgent === "string" && navigator.userAgent.indexOf("Chrome") == -1 && navigator.userAgent.indexOf("Safari") > -1;

// compat.ts
function browser_resolvePath(path, base) {
  if (base === undefined) {
    base = location;
  }
  return new URL(path, base).toString();
}
function browser_getBinaryResponse(path, subResourceHash) {
  const url = new URL(path, location);
  let options = subResourceHash ? { integrity: subResourceHash } : {};
  return { response: fetch(url, options) };
}
async function loadBinaryFile(path, file_sub_resource_hash) {
  const { response, binary } = getBinaryResponse(path, file_sub_resource_hash);
  if (binary) {
    return binary;
  }
  const r = await response;
  if (!r.ok) {
    throw new Error(`Failed to load '${path}': request failed.`);
  }
  return new Uint8Array(await r.arrayBuffer());
}
async function loadLockFile(lockFileURL) {
  let response = await fetch(lockFileURL);
  return await response.json();
}
async function calculateDirname() {
  let err;
  try {
    throw new Error;
  } catch (e) {
    err = e;
  }
  let fileName = error_stack_parser_default.parse(err)[0].fileName;
  const indexOfLastSlash = fileName.lastIndexOf(pathSep);
  if (indexOfLastSlash === -1) {
    throw new Error("Could not extract indexURL path from pyodide module location");
  }
  return fileName.slice(0, indexOfLastSlash);
}
var resolvePath;
resolvePath = browser_resolvePath;
var pathSep;
var getBinaryResponse;
getBinaryResponse = browser_getBinaryResponse;
var loadScript;
if (IN_BROWSER_MAIN_THREAD) {
  loadScript = async (url) => await import(url);
} else if (IN_BROWSER_WEB_WORKER) {
  loadScript = async (url) => {
    try {
      globalThis.importScripts(url);
    } catch (e) {
      if (e instanceof TypeError) {
        await import(url);
      } else {
        throw e;
      }
    }
  };
} else {
  throw new Error("Cannot determine runtime environment");
}

// emscripten-settings.ts
function createSettings(config) {
  const settings = {
    noImageDecoding: true,
    noAudioDecoding: true,
    noWasmDecoding: false,
    preRun: getFileSystemInitializationFuncs(config),
    quit(status, toThrow) {
      settings.exited = { status, toThrow };
      throw toThrow;
    },
    print: config.stdout,
    printErr: config.stderr,
    arguments: config.args,
    API: { config },
    locateFile: (path) => config.indexURL + path,
    instantiateWasm: getInstantiateWasmFunc(config.indexURL)
  };
  return settings;
}
function createHomeDirectory(path) {
  return function(Module) {
    const fallbackPath = "/";
    try {
      Module.FS.mkdirTree(path);
    } catch (e) {
      console.error(`Error occurred while making a home directory '${path}':`);
      console.error(e);
      console.error(`Using '${fallbackPath}' for a home directory instead`);
      path = fallbackPath;
    }
    Module.FS.chdir(path);
  };
}
function setEnvironment(env) {
  return function(Module) {
    Object.assign(Module.ENV, env);
  };
}
function installStdlib(stdlibURL) {
  const stdlibPromise = loadBinaryFile(stdlibURL);
  return (Module) => {
    const pymajor = Module._py_version_major();
    const pyminor = Module._py_version_minor();
    Module.FS.mkdirTree("/lib");
    Module.FS.mkdirTree(`/lib/python${pymajor}.${pyminor}/site-packages`);
    Module.addRunDependency("install-stdlib");
    stdlibPromise.then((stdlib) => {
      Module.FS.writeFile(`/lib/python${pymajor}${pyminor}.zip`, stdlib);
    }).catch((e) => {
      console.error("Error occurred while installing the standard library:");
      console.error(e);
    }).finally(() => {
      Module.removeRunDependency("install-stdlib");
    });
  };
}
function getFileSystemInitializationFuncs(config) {
  let stdLibURL;
  if (config.stdLibURL != null) {
    stdLibURL = config.stdLibURL;
  } else {
    stdLibURL = config.indexURL + "python_stdlib.zip";
  }
  return [
    installStdlib(stdLibURL),
    createHomeDirectory(config.env.HOME),
    setEnvironment(config.env)
  ];
}
function getInstantiateWasmFunc(indexURL) {
  const { binary, response } = getBinaryResponse(indexURL + "pyodide.asm.wasm");
  return function(imports, successCallback) {
    (async function() {
      try {
        let res;
        if (response) {
          res = await WebAssembly.instantiateStreaming(response, imports);
        } else {
          res = await WebAssembly.instantiate(await binary, imports);
        }
        const { instance, module } = res;
        if (typeof WasmOffsetConverter !== "undefined") {
          wasmOffsetConverter = new WasmOffsetConverter(wasmBinary, module);
        }
        successCallback(instance, module);
      } catch (e) {
        console.warn("wasm instantiation failed!");
        console.warn(e);
      }
    })();
    return {};
  };
}

// version.ts
var version = "0.26.2";

// pyodide.ts
async function loadPyodide(options = {}) {
  let indexURL = options.indexURL || await calculateDirname();
  indexURL = resolvePath(indexURL);
  if (!indexURL.endsWith("/")) {
    indexURL += "/";
  }
  options.indexURL = indexURL;
  const default_config = {
    fullStdLib: false,
    jsglobals: globalThis,
    stdin: globalThis.prompt ? globalThis.prompt : undefined,
    lockFileURL: indexURL + "pyodide-lock.json",
    args: [],
    env: {},
    packageCacheDir: indexURL,
    packages: [],
    enableRunUntilComplete: false,
    checkAPIVersion: true
  };
  const config = Object.assign(default_config, options);
  config.env.HOME ??= "/home/pyodide";
  config.env.PYTHONINSPECT ??= "1";
  const emscriptenSettings = createSettings(config);
  const API = emscriptenSettings.API;
  API.lockFilePromise = loadLockFile(config.lockFileURL);
  if (typeof _createPyodideModule !== "function") {
    const scriptSrc = `${config.indexURL}pyodide.asm.js`;
    await loadScript(scriptSrc);
  }
  let snapshot = undefined;
  if (options._loadSnapshot) {
    const snp = await options._loadSnapshot;
    if (ArrayBuffer.isView(snp)) {
      snapshot = snp;
    } else {
      snapshot = new Uint8Array(snp);
    }
    emscriptenSettings.noInitialRun = true;
    emscriptenSettings.INITIAL_MEMORY = snapshot.length;
  }
  const Module = await _createPyodideModule(emscriptenSettings);
  if (emscriptenSettings.exited) {
    throw emscriptenSettings.exited.toThrow;
  }
  if (options.pyproxyToStringRepr) {
    API.setPyProxyToStringMethod(true);
  }
  if (API.version !== version && config.checkAPIVersion) {
    throw new Error(`Pyodide version does not match: '${version}' <==> '${API.version}'. If you updated the Pyodide version, make sure you also updated the 'indexURL' parameter passed to loadPyodide.`);
  }
  Module.locateFile = (path) => {
    throw new Error("Didn't expect to load any more file_packager files!");
  };
  let snapshotConfig = undefined;
  if (snapshot) {
    snapshotConfig = API.restoreSnapshot(snapshot);
  }
  const pyodide = API.finalizeBootstrap(snapshotConfig);
  API.sys.path.insert(0, API.config.env.HOME);
  API._pyodide.set_excepthook();
  await API.packageIndexReady;
  API.initializeStreams(config.stdin, config.stdout, config.stderr);
  return pyodide;
}
export {
  version,
  loadPyodide
};

//# debugId=6380E7864B9BC62F64756E2164756E21
//# sourceMappingURL=pyodide.js.map
